/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * use the Tiny2313 to control the Medion 80326 LCD TV
 *
 * Copyright (c) 2010-2016 Frank Meyer - frank(at)fli4l.de
 *
 *  with additions from m.schoeldgen@hotmail.com
 * $Id: irsnd-main-avr.c,v 1.3 2016/09/09 08:01:11 fm Exp $
 *
 * ATTINY2313 @ 8 MHz internal RC      Osc with BODLEVEL 4.3V: lfuse: 0xE2 hfuse: 0xDC efuse: 0xF9
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
#include "irsnd.h"
#ifndef F_CPU
#  error F_CPU unknown
#endif
#include <util/delay.h>
#include <avr/sleep.h>
// medion specific code 
typedef struct MyFlags {
uint8_t		KeyPressed : 1;   // is there a key?
uint8_t		WasRow0 : 1;
uint8_t		WASRow1 : 1;
uint8_t		IsRepeat : 1;
} MyFlags_t;
// commands
#define MEDION_POWER 0x0C
#define MEDION_LEFT 0x15
#define MEDION_RIGHT 0x16
#define MEDION_UP 0x14
#define MEDION_DOWN 0x13
#define MEDION_HELP 0x12
#define MEDION_VOLUP 0x10
#define MEDION_VOLDN 0x11
#define MEDION_AUDIO 0x23
#define MEDION_MUTE 0x0D
#define MEDION_MENU 0x30
#define MEDION_CHUP  0x20
#define MEDION_CHDOWN 0x21
#define MEDION_FLIPSCREEN 0x22
#define MEDION_PIPPOS 0x29
#define MEDION_PICFORMAT  0x2B
#define MEDION_EPG  0x2F
#define MEDION_FAVLIST 0x34
#define MEDION_OK 0x35
#define MEDION_SOURCE 0x38
#define MEDION_PAGE_UP 0x37
#define MEDION_PAGE_DN 0x36
#define MEDION_TTXT 0x3C
#define MEDION_TVRADIO 0x3F
#define MEDION_ZERO 0x40
#define MEDION_ONE 0x41
#define MEDION_TWO 0x42
#define MEDION_THREE 0x43
#define MEDION_FOUR 0x44
#define MEDION_FIVE 0x45
#define MEDION_SIX  0x46
#define MEDION_SEVEN 0x47
#define MEDION_EIGHT 0x48
#define MEDION_NINE 0x49


// guessed from lirc
//#define MEDION_MENU 0x0052
#define MEDION_EXIT 0x31

#define MEDION_NONE 0x00
// matrix
//
// PB	0		1		2		3		4		5		6		7
//PD0	PWR		HELP	MUTE	VOL+	UP		OK		DWN		VOL-
//PD1   G		H		E		F		C		D		A		B

// output port is declared but IRSND will take care of it
#define IR_OUT PD5 
#define ROW_0 PD0
#define ROW_1 PD1
#define WAS_R0 PD2
#define WAS_R1 PD3
#define SPARE_D PD4

/* VARIABLES */
volatile MyFlags_t MedionFlags;
volatile uint8_t PressedKey; //  
IRMP_DATA irmp_data;

void init_ports(void) {
// set inputs
 DDRB = 0;   // set column all to input
 PORTB |= 0x0FF;   // pullups
// set outputs
DDRD = 0x7F;
DDRA = 0x07;
// DDRD = ((1 << ROW_0) | (1 << ROW_1) | (1 << WAS_R0) | (1 <<WAS_R1) | (1 << IR_OUT) | (1 << SPARE_D)); 
PORTD &= ~((1 << ROW_0) | (1 << ROW_1) | (1 << WAS_R1) | (1 << WAS_R0)); // set low
// set PCINT
 PCMSK = 0x0FF;    // all 8 inputs fire the PCINT
 EIFR = (1 << PCIF);   // clr pendings
 GIMSK |= (1 << PCIE);
}

// Keyboard scanner 
void CheckKeys(void) {
GIMSK &= ~(1 << PCIE);  // block isr
DDRD |= (1 << ROW_0); PORTD &= ~(1 << ROW_0); // select row 0 for output and pull it low
DDRD &= ~(1 << ROW_1) ; PORTD |= (1 << ROW_1); // set row 1 to input and pullup
_delay_ms(5);  // settle key
uint8_t inp = PINB;
if (inp != 0xFF) {
	MedionFlags.KeyPressed = TRUE;
	switch (inp) {
// line 0
		case 0x0FE :  PressedKey = MEDION_POWER; break;
// line 1
		case 0x0FD :  PressedKey = MEDION_MENU; break;
// line 2
		case 0x0FB :  PressedKey = MEDION_SOURCE; break;
// line 3
		case 0x0F7 :  PressedKey = MEDION_RIGHT; break;
// line 4
		case 0x0EF :  PressedKey = MEDION_UP; break;
// line 5
		case 0x0DF :  PressedKey = MEDION_OK; break;
// line 6
		case 0x0BF :  PressedKey = MEDION_DOWN; break;
// line 7
		case 0x07F :  PressedKey = MEDION_LEFT; break;
		} // switch

 	PORTD |= (1 << WAS_R0);  // debug
  } else {
	DDRD |= (1 << ROW_1); PORTD &= ~(1 << ROW_1); // select row 1 for output and low
	DDRD &= ~(1 << ROW_0); PORTD |= (1 << ROW_0); // set row 0 to input and pullup
	_delay_ms(5);
	inp = PINB;
	if (inp != 0xFF) {
		MedionFlags.KeyPressed = TRUE;
// B O A R D  L A Y O U T 
/*---------------------------
   PWR				  MUTE
   			UP
	   VOL- OK	VOL+
			DWN       MENU
  A		C		E		G
  B		D		F		H
  ---------------------------*/
		switch (inp) {
// key G
			case 0x0FE :  PressedKey = MEDION_PICFORMAT; break;
// key H
			case 0x0FD :  PressedKey = MEDION_EPG; break;
// key E
			case 0x0FB :  PressedKey = MEDION_PAGE_UP; break;
// key F
			case 0x0F7 :  PressedKey = MEDION_PAGE_DN; break;
// key C
			case 0x0EF :  PressedKey = MEDION_CHUP; break;
// key D
			case 0x0DF :  PressedKey = MEDION_CHDOWN; break;
// key A
			case 0x0BF :  PressedKey = MEDION_VOLUP; break;
// key B
			case 0x07F :  PressedKey = MEDION_VOLDN; break;
			} // switch
    	PORTD |= ( 1 << WAS_R1);    // debug
    } else {
		MedionFlags.KeyPressed = FALSE;
		MedionFlags.IsRepeat = FALSE;
		PORTD &= ~((1 << WAS_R0) | (1 << WAS_R1));  // reset debug
		}		 
	}  // else
	
DDRD |= ((1 << ROW_0) | (1 << ROW_1)); // both rows output and low for IRQ firing
PORTD &= ~((1 << ROW_0) | (1 << ROW_1));

EIFR = (1 << PCIF);     // clr pendings from port playing
GIMSK |= (1 << PCIE);  // enable ISR
} 
/* here we end up when a key is pressed and the thing was sleeping */
ISR(PCINT_vect) {
CheckKeys();
}
/* send the IR Code with fixed protocol and the command code */
void Flash_IR(uint8_t command){
    irmp_data.protocol = IRMP_RC5_PROTOCOL;      // use RC5 protocol
    irmp_data.address  = 0x0000;                 // set address to 0x0000 for TV
    irmp_data.command  = 0x0000 + command;                // set command 
    irmp_data.flags    = 0;                      // don't repeat frame
    irsnd_send_data (&irmp_data, TRUE);        // send frame, wait for completion
/* autorepeat active ? If yes, wait 600ms */
	if (!MedionFlags.IsRepeat) {
		MedionFlags.IsRepeat = TRUE;
		_delay_ms(600);
	}
	_delay_ms(100);
	MedionFlags.KeyPressed = FALSE;
}
// end medion specific code
// IRSND starts here
void
timer1_init (void)
{
#if defined (__AVR_ATtiny45__) || defined (__AVR_ATtiny85__)                // ATtiny45 / ATtiny85:
    OCR1C   =  (F_CPU / F_INTERRUPTS / 4) - 1;                              // compare value: 1/15000 of CPU frequency, presc = 4
    TCCR1   = (1 << CTC1) | (1 << CS11) | (1 << CS10);                      // switch CTC Mode on, set prescaler to 4
#else                                                                       // ATmegaXX:
    OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                  // compare value: 1/15000 of CPU frequency
    TCCR1B  = (1 << WGM12) | (1 << CS10);                                   // switch CTC Mode on, set prescaler to 1
#endif

#ifdef TIMSK1
    TIMSK1  = 1 << OCIE1A;                                                  // OCIE1A: Interrupt by timer compare
#else
    TIMSK   = 1 << OCIE1A;                                                  // OCIE1A: Interrupt by timer compare
#endif
}

#ifdef TIM1_COMPA_vect                                                      // ATtiny84
#define COMPA_VECT  TIM1_COMPA_vect
#else
#define COMPA_VECT  TIMER1_COMPA_vect                                       // ATmega
#endif

/*------------------------------------------------------
 * timer 1 compare handler, called every 1/10000 sec
 *------------------------------------------------------
 */
ISR(COMPA_VECT)           // Timer1 output compare A interrupt service routine, called every 1/15000 sec
{
    (void) irsnd_ISR();   // call irsnd ISR
    // call other timer interrupt routines here...
}

/*-------------------------
 * MAIN: main routine
 *-------------------------
 */
int
main (void)
{
// preset variables 
	PressedKey = 0;
	MedionFlags.KeyPressed = FALSE;
	MedionFlags.IsRepeat = FALSE;
	MCUSR &= ~(1<<WDRF);
/* Write logical one to WDCE and WDE */
	WDTCSR |= (1<<WDCE) | (1<<WDE);
/* Turn off WDT */
	WDTCSR = 0x00;    
	init_ports();
// save energy - its better for Berlin although we go to sleep in a second
	ACSR = (1 << ACD);
// 
    irsnd_init();                                                           // initialize irsnd
    timer1_init();                                                          // initialize timer
    EIFR |= (1 << PCIF);		// clr pendings
// let them run
    sei ();                                                                 // enable interrupts
    for (;;)
// poll the PCINT Flags and call accordingly
    {
		while (MedionFlags.KeyPressed) {
    	 	Flash_IR(PressedKey);
			CheckKeys();
		 }
    set_sleep_mode(SLEEP_MODE_PWR_DOWN);  // set power down mode
	sleep_mode();
    }
}
